@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*
- * Copyright (C) 2001 Mikael Hallendal <micke@codefactory.se>
+ * Copyright (C) 2001-2002 Mikael Hallendal <micke@codefactory.se>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -33,39 +33,40 @@
 #include "yelp-util.h"
 #include "yelp-scrollkeeper.h"
 
-static gboolean   ys_trim_empty_branches  (xmlNode              *cl_node);
-static gboolean   ys_tree_empty           (xmlNode              *cl_node);
+#define d(x)
 
-static gboolean   ys_parse_books          (GNode                *tree,
-					   xmlDoc               *doc);
-static gboolean   ys_parse_section        (GNode                *parent,
-					   xmlNode              *xml_node);
-static void       ys_parse_doc            (GNode                *parent,
-					   xmlNode              *xml_node,
-					   gchar                *docid);
-static void       ys_parse_toc_section    (GNode                *parent,
-					   xmlNode              *xml_node,
-					   const gchar          *base_uri);
-static gchar *    ys_get_xml_docpath      (const gchar          *command,
-					   const gchar          *argument);
+static gboolean   scrollkeeper_trim_empty_branches (xmlNode      *cl_node);
+static gboolean   scrollkeeper_tree_empty          (xmlNode      *cl_node);
 
-static gchar *    ys_strip_scheme         (gchar                *original_uri,
-					   gchar               **scheme);
+static gboolean   scrollkeeper_parse_books         (GNode        *tree,
+						    xmlDoc       *doc);
+static gboolean   scrollkeeper_parse_section       (GNode        *parent,
+						    xmlNode      *xml_node);
+static void       scrollkeeper_parse_doc           (GNode        *parent,
+						    xmlNode      *xml_node,
+						    gchar        *docid);
+static void       scrollkeeper_parse_toc_section   (GNode        *parent,
+						    xmlNode      *xml_node,
+						    const gchar  *base_uri);
+static gchar *    scrollkeeper_get_xml_docpath     (const gchar  *command,
+						    const gchar  *argument);
+static gchar *    scrollkeeper_strip_scheme        (gchar        *original_uri,
+						    gchar       **scheme);
 
-static gboolean   ys_parse_index          (GList               **index);
-static void       ys_parse_index_file     (GList               **index,
-                                           const gchar          *index_path,
-					   YelpSection          *section);
-static void       ys_parse_index_item     (GList               **index,
-                                           YelpSection          *section,
-					   xmlNode              *node);
+static gboolean   scrollkeeper_parse_index         (GList       **index);
+static void       scrollkeeper_parse_index_file    (GList       **index,
+						    const gchar  *index_path,
+						    YelpSection  *section);
+static void       scrollkeeper_parse_index_item    (GList       **index,
+						    YelpSection  *section,
+						    xmlNode      *node);
 
 
 static GHashTable *seriesid_hash = NULL;
 static GHashTable *docid_hash    = NULL;
 
 static gboolean
-ys_trim_empty_branches (xmlNode *node)
+scrollkeeper_trim_empty_branches (xmlNode *node)
 {
 	xmlNode  *child;
 	xmlNode  *next;
@@ -79,7 +80,7 @@ ys_trim_empty_branches (xmlNode *node)
 		next = child->next;
 		
 		if (!g_ascii_strcasecmp (child->name, "sect")) {
-			empty = ys_trim_empty_branches (child);
+			empty = scrollkeeper_trim_empty_branches (child);
 			if (empty) {
 				xmlUnlinkNode (child);
 				xmlFreeNode (child);
@@ -98,7 +99,7 @@ ys_trim_empty_branches (xmlNode *node)
 }
 
 static gboolean
-ys_tree_empty (xmlNode *cl_node)
+scrollkeeper_tree_empty (xmlNode *cl_node)
 {
 	xmlNode  *node, *next;
 	gboolean  ret_val;
@@ -111,7 +112,7 @@ ys_tree_empty (xmlNode *cl_node)
 
 		if (!g_ascii_strcasecmp (node->name, "sect") &&
 		    node->xmlChildrenNode->next != NULL) {
-			ret_val = ys_tree_empty (
+			ret_val = scrollkeeper_tree_empty (
 				node->xmlChildrenNode->next);
 
 			if (!ret_val) {
@@ -128,7 +129,7 @@ ys_tree_empty (xmlNode *cl_node)
 }
 
 static gboolean
-ys_parse_books (GNode *tree, xmlDoc *doc)
+scrollkeeper_parse_books (GNode *tree, xmlDoc *doc)
 {
 	xmlNode  *node;
 	gboolean  success;
@@ -143,15 +144,15 @@ ys_parse_books (GNode *tree, xmlDoc *doc)
 		g_warning ("Invalid ScrollKeeper XML Contents List!");
 		return FALSE;
 	}
-	
+
 	book_node = g_node_append_data (tree, 
 					yelp_section_new (YELP_SECTION_CATEGORY,
-							  "scrollkeeper", NULL,
-							  NULL, NULL));
-       
+							  "scrollkeeper", 
+							  NULL));
+
 	for (node = node->xmlChildrenNode; node; node = node->next) {
 		if (!g_ascii_strcasecmp (node->name, "sect")) {
-			success = ys_parse_section (book_node, node);
+			success = scrollkeeper_parse_section (book_node, node);
 		}
 	}
 
@@ -159,7 +160,7 @@ ys_parse_books (GNode *tree, xmlDoc *doc)
 }
 
 static gboolean
-ys_parse_section (GNode *parent, xmlNode *xml_node)
+scrollkeeper_parse_section (GNode *parent, xmlNode *xml_node)
 {
 	xmlNode *cur;
 	xmlChar *xml_str;
@@ -186,12 +187,11 @@ ys_parse_section (GNode *parent, xmlNode *xml_node)
 	
 	node = g_node_append_data (parent, 
 				   yelp_section_new (YELP_SECTION_CATEGORY, 
-						     name, NULL,
-						     NULL, NULL));
+						     name, NULL));
 
 	for (cur = xml_node->xmlChildrenNode; cur; cur = cur->next) {
 		if (!g_ascii_strcasecmp (cur->name, "sect")) {
-			ys_parse_section (node, cur);
+			scrollkeeper_parse_section (node, cur);
 		}
 		else if (!g_ascii_strcasecmp (cur->name, "doc")) {
 
@@ -201,7 +201,7 @@ ys_parse_section (GNode *parent, xmlNode *xml_node)
 				xmlFree (xml_str);
 			}
 			
-			ys_parse_doc (node, cur, docid);
+			scrollkeeper_parse_doc (node, cur, docid);
 		}
 	}
 	
@@ -209,7 +209,7 @@ ys_parse_section (GNode *parent, xmlNode *xml_node)
 }
 
 static void
-ys_parse_doc (GNode *parent, xmlNode *xml_node, gchar *docid)
+scrollkeeper_parse_doc (GNode *parent, xmlNode *xml_node, gchar *docid)
 {
 	xmlNode *cur;
 	xmlChar *xml_str;
@@ -220,6 +220,7 @@ ys_parse_doc (GNode *parent, xmlNode *xml_node, gchar *docid)
 	gchar   *docsource;
 	gchar   *docseriesid;
 	GNode   *node;
+	YelpURI *uri;
 
 	docseriesid = NULL;
 	for (cur = xml_node->xmlChildrenNode; cur; cur = cur->next) {
@@ -235,7 +236,7 @@ ys_parse_doc (GNode *parent, xmlNode *xml_node, gchar *docid)
 		}
 		else if (!g_ascii_strcasecmp (cur->name, "docsource")) {
 			xml_str = xmlNodeGetContent (cur);
-			docsource = ys_strip_scheme (xml_str, NULL);
+			docsource = scrollkeeper_strip_scheme (xml_str, NULL);
 			link    = g_strconcat ("ghelp:", docsource, NULL);
 			xmlFree (xml_str);
 		}
@@ -251,10 +252,12 @@ ys_parse_doc (GNode *parent, xmlNode *xml_node, gchar *docid)
 		}
 	}
 
+	uri = yelp_uri_new (link);
+
 	node = g_node_append_data (parent, 
                                    yelp_section_new (YELP_SECTION_DOCUMENT,
-                                                     title, link,
-                                                     NULL, NULL));
+                                                     title, uri));
+	yelp_uri_unref (uri);
 
 	if (docseriesid) {
 		g_hash_table_insert (seriesid_hash, docseriesid, node);
@@ -270,15 +273,17 @@ ys_parse_doc (GNode *parent, xmlNode *xml_node, gchar *docid)
 }
 
 static void
-ys_parse_toc_section (GNode       *parent,
-		      xmlNode     *xml_node, 
-		      const gchar *base_uri)
+scrollkeeper_parse_toc_section (GNode       *parent,
+				xmlNode     *xml_node, 
+				const gchar *base_uri)
 {
-	gchar       *name;
-	gchar       *link;
-	xmlNode     *next_child;
-	xmlChar     *xml_str;
-        GNode       *node;
+	gchar   *name;
+	gchar   *link;
+	xmlNode *next_child;
+	xmlChar *xml_str;
+        GNode   *node;
+	gchar   *str_uri;
+	YelpURI *uri;
 
 	next_child = xml_node->xmlChildrenNode;
 	
@@ -288,7 +293,7 @@ ys_parse_toc_section (GNode       *parent,
 		return;
 	}
 
-	g_strchomp (name);
+	g_strstrip (name);
 
 	xml_str = xmlGetProp (xml_node, "linkid");
 	
@@ -297,38 +302,38 @@ ys_parse_toc_section (GNode       *parent,
 		g_strchomp (link);
 		xmlFree (xml_str);
 	}
-	
-/* 	if (link) { */
-/* 		uri = gnome_vfs_uri_resolve_relative (base_uri, link); */
-/* 	} else { */
-/* 	} */
 
+	str_uri = g_strconcat (base_uri, link, NULL);
+	uri     = yelp_uri_new (str_uri);
+	g_free (str_uri);
+	
         node = g_node_append_data (parent, 
                                    yelp_section_new (YELP_SECTION_DOCUMENT_SECTION,
-                                                     name, 
-                                                     base_uri,
-                                                     link, NULL));
+                                                     name, uri));
+	yelp_uri_unref (uri);
 	
 	for (; next_child != NULL; next_child = next_child->next) {
 		if (!g_ascii_strncasecmp (next_child->name, "tocsect", 7)) {
-			ys_parse_toc_section (node, next_child, base_uri);
+			scrollkeeper_parse_toc_section (node, next_child, base_uri);
 		}
 	}
 }
 
 static gchar *
-ys_get_xml_docpath (const gchar *command, const gchar *argument)
+scrollkeeper_get_xml_docpath (const gchar *command, const gchar *argument)
 {
 	gboolean  success;
 	gchar    *full_command;
 	gchar    *xml_location = NULL;
+	gchar    *std_err;
 	
 	full_command = g_strconcat (command, " ", argument, NULL);
 
 	success = g_spawn_command_line_sync (full_command, &xml_location,
-					     NULL, NULL, NULL);
+					     &std_err, NULL, NULL);
 
 	g_free (full_command);
+	g_free (std_err);
 	
 	if (!success) {
 		g_warning ("Didn't successfully run command: '%s %s'", 
@@ -347,7 +352,7 @@ ys_get_xml_docpath (const gchar *command, const gchar *argument)
 }
 
 static gchar *
-ys_strip_scheme(gchar *original_uri, gchar **scheme)
+scrollkeeper_strip_scheme(gchar *original_uri, gchar **scheme)
 {
 	gchar *new_uri;
 	gchar *point;
@@ -372,7 +377,7 @@ ys_strip_scheme(gchar *original_uri, gchar **scheme)
 }
 
 static gboolean
-ys_parse_index (GList **index)
+scrollkeeper_parse_index (GList **index)
 {
 	gchar                   *sk_data_dir = NULL;
 	gchar                   *index_dir;
@@ -382,8 +387,8 @@ ys_parse_index (GList **index)
 	GNode                   *node;
 	YelpSection             *section;
 	
-	sk_data_dir = ys_get_xml_docpath ("scrollkeeper-config",
-					  "--pkglocalstatedir");
+	sk_data_dir = scrollkeeper_get_xml_docpath ("scrollkeeper-config",
+						    "--pkglocalstatedir");
 
 	index_dir = g_strdup_printf ("%s/index", sk_data_dir);
 
@@ -393,7 +398,6 @@ ys_parse_index (GList **index)
 					   GNOME_VFS_FILE_INFO_DEFAULT);
 
 	if (result != GNOME_VFS_OK) {
-		g_warning ("Error opening index directory: %s\n", index_dir);
 		return FALSE;
 	}
 
@@ -409,7 +413,7 @@ ys_parse_index (GList **index)
 			
 			section = YELP_SECTION (node->data);
 			
-			ys_parse_index_file (index, index_path, section);
+			scrollkeeper_parse_index_file (index, index_path, section);
 
 			g_free (index_path);
 		}
@@ -417,14 +421,15 @@ ys_parse_index (GList **index)
 	
 	g_free (index_dir);
 	gnome_vfs_file_info_unref (file_info);
-
+	gnome_vfs_directory_close (dir);
+	
 	return TRUE;
 }
 
 static void
-ys_parse_index_file (GList       **index, 
-		     const gchar  *index_path,
-		     YelpSection  *section)
+scrollkeeper_parse_index_file (GList       **index, 
+			       const gchar  *index_path,
+			       YelpSection  *section)
 {
 	xmlDoc  *doc;
 	xmlNode *node;
@@ -443,21 +448,19 @@ ys_parse_index_file (GList       **index,
 		for (node = node->xmlChildrenNode; node; node = node->next) {
 			if (!g_ascii_strcasecmp (node->name, "indexitem")) {
 				
-				ys_parse_index_item (index, section, node);
+				scrollkeeper_parse_index_item (index, section, node);
 			}
 		}
 	}
 }
 
 static void
-ys_parse_index_item (GList **index, YelpSection *section, xmlNode *node)
+scrollkeeper_parse_index_item (GList **index, YelpSection *section, xmlNode *node)
 {
 	xmlNode     *cur;
 	xmlChar     *title = NULL;
 	xmlChar     *link  = NULL;
-	YelpSection *index_section;
 	xmlChar     *xml_str;
-	gchar       *index_uri;
 	
 	for (cur = node->xmlChildrenNode; cur; cur = cur->next) {
 		if (!g_ascii_strcasecmp (cur->name, "title")) {
@@ -478,46 +481,53 @@ ys_parse_index_item (GList **index, YelpSection *section, xmlNode *node)
 			xmlFree (xml_str);
 		}
 		else if (!g_ascii_strcasecmp (cur->name, "indexitem")) {
-			ys_parse_index_item (index, section, cur);
+			scrollkeeper_parse_index_item (index, section, cur);
 		}
 	}
 
 	if (title && link) {
-		index_uri = g_strconcat ("index:", section->uri, NULL);
+		YelpURI     *uri;
+		YelpSection *index_section;
+		
+		uri = yelp_uri_get_relative (section->uri, link);
+		
+		d(g_print ("%s\n", yelp_uri_to_string (uri)));
 		
 		index_section = yelp_section_new (YELP_SECTION_INDEX,
-						  title, index_uri,
-						  link, NULL);
+						  title, uri);
+		
+		yelp_uri_unref (uri);
 		
-		g_free (index_uri);
-
 		*index = g_list_prepend (*index, index_section);
 		
 		g_free (title);
 		g_free (link);
+		
+		title = link = NULL;
 	}
 }
 
 gboolean
 yelp_scrollkeeper_init (GNode *tree, GList **index)
 {
-       gchar       *docpath;
-       xmlDoc      *doc;
-       const GList *node;
+	gchar       *docpath;
+	xmlDoc      *doc;
+	const GList *node;
         
-       g_return_val_if_fail (tree != NULL, FALSE);
+	g_return_val_if_fail (tree != NULL, FALSE);
 
-       seriesid_hash = g_hash_table_new_full (g_str_hash,
-					      g_str_equal,
-					      g_free, NULL);
+	seriesid_hash = g_hash_table_new_full (g_str_hash,
+					       g_str_equal,
+					       g_free, NULL);
 
-       docid_hash = g_hash_table_new_full (g_str_hash, g_str_equal, 
-					   g_free, NULL);
+	docid_hash = g_hash_table_new_full (g_str_hash, g_str_equal, 
+					    g_free, NULL);
        
-       doc = NULL;
-       for (node = gnome_i18n_get_language_list ("LC_MESSAGES"); node; node = node->next) {
-	       docpath = ys_get_xml_docpath ("scrollkeeper-get-content-list",
-					     node->data);
+	doc = NULL;
+
+	for (node = gnome_i18n_get_language_list ("LC_MESSAGES"); node; node = node->next) {
+		docpath = scrollkeeper_get_xml_docpath ("scrollkeeper-get-content-list",
+							node->data);
 
 		if (docpath) {
 			doc = xmlParseFile (docpath);
@@ -525,7 +535,7 @@ yelp_scrollkeeper_init (GNode *tree, GList **index)
 		}
 
 		if (doc) {
-			if (doc->xmlRootNode && !ys_tree_empty(doc->xmlRootNode->xmlChildrenNode)) {
+			if (doc->xmlRootNode && !scrollkeeper_tree_empty(doc->xmlRootNode->xmlChildrenNode)) {
 				break;
 			} else {
 				xmlFreeDoc (doc);
@@ -535,14 +545,14 @@ yelp_scrollkeeper_init (GNode *tree, GList **index)
 	}
 		
 	if (doc) {
-		ys_trim_empty_branches (doc->xmlRootNode);
+		scrollkeeper_trim_empty_branches (doc->xmlRootNode);
 
-		ys_parse_books (tree, doc);
+		scrollkeeper_parse_books (tree, doc);
 		
 		xmlFreeDoc (doc);
 	}        
 
-	ys_parse_index (index);
+	scrollkeeper_parse_index (index);
 
         return TRUE;
 }
@@ -566,8 +576,8 @@ yelp_scrollkeeper_get_toc_tree (const gchar *docpath)
 
         tree = g_node_new (NULL);
 	
- 	toc_file = ys_get_xml_docpath ("scrollkeeper-get-toc-from-docpath",
- 				       docpath);
+ 	toc_file = scrollkeeper_get_xml_docpath ("scrollkeeper-get-toc-from-docpath",
+						 docpath);
 
 	if (toc_file) {
 		doc = xmlParseFile (toc_file);
@@ -589,7 +599,7 @@ yelp_scrollkeeper_get_toc_tree (const gchar *docpath)
 	full_path = g_strconcat ("ghelp:", docpath, NULL);
 
 	for (; xml_node != NULL; xml_node = xml_node->next) {
-		ys_parse_toc_section (tree, xml_node, full_path);
+		scrollkeeper_parse_toc_section (tree, xml_node, full_path);
 	}
 
 	g_free (full_path);
