@@ -30,6 +30,7 @@
 #include <gconf/gconf-client.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <libgnomeui/gnome-about.h>
+#include <libgnomeui/gnome-stock-icons.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnome/gnome-url.h>
 #include <string.h>
@@ -55,64 +56,63 @@ static GConfEnumStringPair toolbar_styles[] = {
         { GTK_TOOLBAR_BOTH, "both" }
 };
 
-static void        yw_init		       (YelpWindow          *window);
-static void        yw_class_init	       (YelpWindowClass     *klass);
+static void        window_init		          (YelpWindow        *window);
+static void        window_class_init	          (YelpWindowClass   *klass);
 
-static void        yw_populate                 (YelpWindow          *window);
+static void        window_populate                (YelpWindow        *window);
 
-static gboolean    yw_handle_url               (YelpWindow          *window,
-						const gchar         *url);
-static void        yw_url_selected_cb          (gpointer             view,
-						char                *url,
-						char                *base_url,
-						gboolean             handled,
-						YelpWindow          *window);
-static void        yw_toggle_history_back      (YelpHistory         *history,
-						gboolean             sensitive,
-						YelpWindow          *window);
+static gboolean    window_handle_uri              (YelpWindow        *window,
+						    YelpURI          *uri);
+static void        window_uri_selected_cb         (gpointer           view,
+						    YelpURI          *uri,
+						    gboolean          handled,
+						    YelpWindow       *window);
+static void        window_toggle_history_back     (YelpHistory       *history,
+						    gboolean          sensitive,
+						    YelpWindow       *window);
 
-static void        yw_toggle_history_forward   (YelpHistory         *history,
-						gboolean             sensitive,
-						YelpWindow          *window);
+static void        window_toggle_history_forward  (YelpHistory       *history,
+						   gboolean           sensitive,
+						   YelpWindow        *window);
 
-static void        yw_history_action           (YelpWindow          *window,
-						YelpHistoryAction    action);
-static void        yw_back_button_clicked      (GtkWidget           *button,
-						YelpWindow          *window);
-static void        yw_forward_button_clicked   (GtkWidget           *button,
-						YelpWindow          *window);
-static void        yw_home_button_clicked      (GtkWidget           *button,
-						YelpWindow          *window);
-static void        yw_index_button_clicked     (GtkWidget           *button,
-						YelpWindow          *window);
-static void        yw_new_window_cb            (gpointer             data,
-						guint                section,
-						GtkWidget           *widget);
-static void        yw_close_window_cb          (gpointer             data,
-						guint                section,
-						GtkWidget           *widget);
-static void        yw_history_go_cb            (gpointer             data,
-						guint                section,
-						GtkWidget           *widget);
-static void        yw_go_home_cb               (gpointer             data,
-						guint                section,
-						GtkWidget           *widget);
-static void        yw_go_index_cb              (gpointer             data,
-						guint                section,
-						GtkWidget           *widget);
-static void        yw_about_cb                 (gpointer             data,
-						guint                section,
-						GtkWidget           *widget);
-static GtkWidget * yw_create_toolbar           (YelpWindow          *window);
+static void        window_history_action          (YelpWindow        *window,
+						   YelpHistoryAction  action);
+static void        window_back_button_clicked     (GtkWidget        *button,
+						   YelpWindow       *window);
+static void        window_forward_button_clicked  (GtkWidget        *button,
+						   YelpWindow       *window);
+static void        window_home_button_clicked     (GtkWidget        *button,
+						   YelpWindow       *window);
+static void        window_index_button_clicked    (GtkWidget        *button,
+						   YelpWindow       *window);
+static void        window_new_window_cb           (gpointer          data,
+						   guint             section,
+						   GtkWidget        *widget);
+static void        window_close_window_cb         (gpointer          data,
+						   guint             section,
+						   GtkWidget        *widget);
+static void        window_history_go_cb           (gpointer          data,
+						   guint             section,
+						   GtkWidget        *widget);
+static void        window_go_home_cb              (gpointer          data,
+						   guint             section,
+						   GtkWidget        *widget);
+static void        window_go_index_cb             (gpointer          data,
+						   guint             section,
+						   GtkWidget        *widget);
+static void        window_about_cb                (gpointer          data,
+						   guint             section,
+						   GtkWidget        *widget);
+static GtkWidget * window_create_toolbar          (YelpWindow       *window);
 
-static void        yw_toolbar_style_changed_cb (GConfClient         *client,
-						guint                cnxn_id,
-						GConfEntry          *entry,
-						gpointer             data);
+static void        window_toolbar_style_changed_cb (GConfClient     *client,
+						    guint            cnxn_id,
+						    GConfEntry      *entry,
+						    gpointer         data);
 
 #if 0
-static void        yw_remove_notify_cb          (GtkObject          *obj, 
-						 gpointer            data);
+static void        window_remove_notify_cb        (GtkObject        *obj, 
+						   gpointer          data);
 #endif
 
 enum {
@@ -134,7 +134,7 @@ struct _YelpWindowPriv {
 
 	GtkWidget      *notebook;
 
-	GtkWidget      *toc_view;
+	YelpViewTOC    *toc_view;
 	GtkWidget      *content_view;
 	GtkWidget      *index_view;
 	
@@ -150,16 +150,16 @@ struct _YelpWindowPriv {
 
 static GtkItemFactoryEntry menu_items[] = {
 	{N_("/_File"),              NULL,         0,                  0,                           "<Branch>"},
-	{N_("/File/_New window"),   NULL,         yw_new_window_cb,   0,                           "<StockItem>", GTK_STOCK_NEW     },
-	{N_("/File/_Close window"), NULL,         yw_close_window_cb, 0,                           "<StockItem>", GTK_STOCK_CLOSE   },
-/*	{N_("/File/_Quit"),         NULL,         yw_exit_cb,         0,                           "<StockItem>", GTK_STOCK_QUIT    }, */
+	{N_("/File/_New window"),   NULL,         window_new_window_cb,   0,                           "<StockItem>", GTK_STOCK_NEW     },
+	{N_("/File/_Close window"), NULL,         window_close_window_cb, 0,                           "<StockItem>", GTK_STOCK_CLOSE   },
+/*	{N_("/File/_Quit"),         NULL,         window_exit_cb,         0,                           "<StockItem>", GTK_STOCK_QUIT    }, */
 	{N_("/_Go"),                NULL,         0,                  0,                           "<Branch>"},
-	{N_("/Go/_Back"),           NULL,         yw_history_go_cb,   YELP_WINDOW_ACTION_BACK,     "<StockItem>", GTK_STOCK_GO_BACK    },
-	{N_("/Go/_Forward"),        NULL,         yw_history_go_cb,   YELP_WINDOW_ACTION_FORWARD,  "<StockItem>", GTK_STOCK_GO_FORWARD },
-	{N_("/Go/_Home"),           NULL,         yw_go_home_cb,      0,                           "<StockItem>", GTK_STOCK_HOME },
-	{N_("/Go/_Index"),          NULL,         yw_go_index_cb,     0,                           "<StockItem>", GTK_STOCK_INDEX },
+	{N_("/Go/_Back"),           NULL,         window_history_go_cb,   YELP_WINDOW_ACTION_BACK,     "<StockItem>", GTK_STOCK_GO_BACK    },
+	{N_("/Go/_Forward"),        NULL,         window_history_go_cb,   YELP_WINDOW_ACTION_FORWARD,  "<StockItem>", GTK_STOCK_GO_FORWARD },
+	{N_("/Go/_Home"),           NULL,         window_go_home_cb,      0,                           "<StockItem>", GTK_STOCK_HOME },
+	{N_("/Go/_Index"),          NULL,         window_go_index_cb,     0,                           "<StockItem>", GTK_STOCK_INDEX },
 	{N_("/_Help"),              NULL,         0,                  0,                           "<Branch>"},
-	{N_("/Help/_About"),        NULL,         yw_about_cb,        0,                           NULL },
+	{N_("/Help/_About"),        NULL,         window_about_cb,        0,                           "<StockItem>", GNOME_STOCK_ABOUT },
 };
 
 GType
@@ -173,12 +173,12 @@ yelp_window_get_type (void)
                                 sizeof (YelpWindowClass),
                                 NULL,
                                 NULL,
-                                (GClassInitFunc) yw_class_init,
+                                (GClassInitFunc) window_class_init,
                                 NULL,
                                 NULL,
                                 sizeof (YelpWindow),
                                 0,
-                                (GInstanceInitFunc) yw_init,
+                                (GInstanceInitFunc) window_init,
                         };
                 
                 window_type = g_type_register_static (GTK_TYPE_WINDOW,
@@ -190,30 +190,33 @@ yelp_window_get_type (void)
 }
 
 static void
-yw_init (YelpWindow *window)
+window_init (YelpWindow *window)
 {
         YelpWindowPriv *priv;
-       
+	YelpURI        *uri;
+	
         priv = g_new0 (YelpWindowPriv, 1);
         window->priv = priv;
         
-	priv->toc_view    = NULL;
+	priv->toc_view     = NULL;
 	priv->content_view = NULL;
 	priv->index_view   = NULL;
 	priv->view_current = NULL;
 	
 	priv->history = yelp_history_new ();
 
-	yelp_history_goto (priv->history, "toc:");
+	uri = yelp_uri_new ("toc:");
+	yelp_history_goto (priv->history, uri);
+	yelp_uri_unref (uri);
 
 	g_signal_connect (priv->history, 
 			  "back_exists_changed",
-			  G_CALLBACK (yw_toggle_history_back),
+			  G_CALLBACK (window_toggle_history_back),
 			  window);
 
 	g_signal_connect (priv->history, 
 			  "forward_exists_changed",
-			  G_CALLBACK (yw_toggle_history_forward),
+			  G_CALLBACK (window_toggle_history_forward),
 			  window);
 
 	gtk_window_set_default_size (GTK_WINDOW (window), 600, 420);
@@ -222,7 +225,7 @@ yw_init (YelpWindow *window)
 }
 
 static void
-yw_class_init (YelpWindowClass *klass)
+window_class_init (YelpWindowClass *klass)
 {
 	signals[NEW_WINDOW_REQUESTED] =
 		g_signal_new ("new_window_requested",
@@ -236,7 +239,7 @@ yw_class_init (YelpWindowClass *klass)
 }
 
 static void
-yw_populate (YelpWindow *window)
+window_populate (YelpWindow *window)
 {
         YelpWindowPriv *priv;
 	GtkWidget      *toolbar;
@@ -253,7 +256,7 @@ yw_populate (YelpWindow *window)
 
 	accel_group  = gtk_accel_group_new ();
 	priv->item_factory = gtk_item_factory_new (GTK_TYPE_MENU_BAR, 
-					     "<main>", accel_group);
+						   "<main>", accel_group);
 
 	gtk_item_factory_set_translate_func (priv->item_factory, 
 					     (GtkTranslateFunc) gettext,
@@ -279,7 +282,7 @@ yw_populate (YelpWindow *window)
 							 "<main>"),
 			    FALSE, FALSE, 0);
 
-	toolbar         = yw_create_toolbar (window);
+	toolbar         = window_create_toolbar (window);
 
 	priv->notebook  = gtk_notebook_new ();
 
@@ -292,7 +295,8 @@ yw_populate (YelpWindow *window)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 
-	gtk_container_add (GTK_CONTAINER (sw), priv->toc_view);
+	gtk_container_add (GTK_CONTAINER (sw), 
+			   yelp_view_toc_get_widget (priv->toc_view));
 
 	gtk_notebook_insert_page (GTK_NOTEBOOK (priv->notebook),
 				  sw, NULL, PAGE_TOC_VIEW);
@@ -313,7 +317,7 @@ yw_populate (YelpWindow *window)
 }
 
 static gboolean
-yw_handle_url (YelpWindow *window, const gchar *url)
+window_handle_uri (YelpWindow *window, YelpURI *uri)
 {
 	YelpWindowPriv *priv;
 	GError         *error = NULL;
@@ -321,35 +325,37 @@ yw_handle_url (YelpWindow *window, const gchar *url)
 
 	priv = window->priv;
 
-	d(g_print ("Handling URL: %s\n", url));
+	d(g_print ("Handling URL: %s\n", yelp_uri_to_string (uri)));
 
- 	yelp_view_content_stop (YELP_VIEW_CONTENT (priv->content_view));
+	if (yelp_uri_get_type (uri) == YELP_URI_TYPE_TOC) {
+		d(g_print ("[TOC]\n"));
+		
+		yelp_view_toc_open_uri (YELP_VIEW_TOC (priv->toc_view), uri);
 
-	if (strncmp (url, "toc:", 4) == 0) {
-		yelp_view_toc_open_url (YELP_VIEW_TOC (priv->toc_view),
-					url);
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
 					       PAGE_TOC_VIEW);
 		handled = TRUE;
-	} else if (strncmp (url, "man:", 4) == 0 ||
-		   strncmp (url, "info:", 5) == 0 ||
-		   strncmp (url, "ghelp:", 6) == 0 ||
-		   strncmp (url, "path:", 5) == 0) {
+	} 
+	else if (yelp_uri_get_type (uri) == YELP_URI_TYPE_INDEX) {
+		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
+					       PAGE_INDEX_VIEW);
+		handled = TRUE;
+	}
+	else if (yelp_uri_get_type (uri) == YELP_URI_TYPE_MAN ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_INFO ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_DOCBOOK_XML ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_DOCBOOK_SGML ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_HTML ||
+		 yelp_uri_get_type (uri) == YELP_URI_TYPE_PATH) {
+		d(g_print ("[CONTENT]\n"));
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
 					       PAGE_CONTENT_VIEW);
 		yelp_view_content_show_uri (YELP_VIEW_CONTENT (priv->content_view),
-					    url,
+					    uri,
 					    &error);
 		handled = TRUE;
-	} else if (strncmp (url, "index:", 6) == 0) {
-		gtk_notebook_set_current_page (GTK_NOTEBOOK (priv->notebook),
-					       PAGE_INDEX_VIEW);
-		yelp_view_index_show_uri (YELP_VIEW_INDEX (priv->index_view),
-					  url,
-					  &error);
-		handled = TRUE;
 	} else {
-		gnome_url_show (url, &error);
+		gnome_url_show (yelp_uri_to_string (uri), &error);
 		handled = FALSE;
 	}
 
@@ -369,53 +375,37 @@ yw_handle_url (YelpWindow *window, const gchar *url)
 }
 
 static void
-yw_url_selected_cb (gpointer    view,
-		    char       *url,
-		    char       *base_url,
-		    gboolean    handled,
-		    YelpWindow *window)
+window_uri_selected_cb (gpointer    view, 
+			YelpURI    *uri, 
+			gboolean    handled,
+			YelpWindow *window)
 {
 	YelpWindowPriv *priv;
-	gchar          *abs_url = NULL;
-
 
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
-	d(g_print ("url_selected: %s base: %s, handled: %d\n", url, base_url, handled));
+	d(g_print ("uri_selected: %s, handled: %d\n", 
+		   yelp_uri_to_string (uri), handled));
 
 	priv = window->priv;
-	
-	if (url && base_url) {
-		abs_url = yelp_util_resolve_relative_uri (base_url, url);
-		
-		d(g_print ("Link '%s' pressed relative to: %s -> %s\n", 
-			   url,
-			   base_url,
-			   abs_url));
-        } else {
-		if (url) {
-			abs_url = g_strdup (url);
-		}
-		else if (base_url) {
-			abs_url = g_strdup (base_url);
-		}
-        }
 
+	yelp_uri_ref (uri);
+	
 	if (handled) {
-		yelp_history_goto (priv->history, abs_url);
+		yelp_history_goto (priv->history, uri);
 	} else {
-		if (yw_handle_url (window, abs_url)) {
-			yelp_history_goto (priv->history, abs_url);
+		if (window_handle_uri (window, uri)) {
+			yelp_history_goto (priv->history, uri);
 		}
 	}
 
-	g_free (abs_url);
+	yelp_uri_unref (uri);
 }
 
 static void
-yw_toggle_history_back (YelpHistory *history,
-			gboolean     sensitive,
-			YelpWindow  *window)
+window_toggle_history_back (YelpHistory *history,
+			    gboolean     sensitive,
+			    YelpWindow  *window)
 {
 	YelpWindowPriv *priv;
 	GtkWidget      *menu_item;
@@ -433,9 +423,9 @@ yw_toggle_history_back (YelpHistory *history,
 }
 
 static void
-yw_toggle_history_forward (YelpHistory *history,
-			   gboolean     sensitive,
-			   YelpWindow  *window)
+window_toggle_history_forward (YelpHistory *history,
+			       gboolean     sensitive,
+			       YelpWindow  *window)
 {
 	YelpWindowPriv *priv;
 	GtkWidget      *menu_item;
@@ -453,10 +443,10 @@ yw_toggle_history_forward (YelpHistory *history,
 }
 
 static void
-yw_history_action (YelpWindow *window, YelpHistoryAction action)
+window_history_action (YelpWindow *window, YelpHistoryAction action)
 {
 	YelpWindowPriv *priv;
-	const gchar    *url;
+	YelpURI        *uri;
 	
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
@@ -464,63 +454,73 @@ yw_history_action (YelpWindow *window, YelpHistoryAction action)
 
 	switch (action) {
 	case YELP_WINDOW_ACTION_BACK:
-		url = yelp_history_go_back (priv->history);
+		uri = yelp_history_go_back (priv->history);
 		break;
 	case YELP_WINDOW_ACTION_FORWARD:
-		url = yelp_history_go_forward (priv->history);
+		uri = yelp_history_go_forward (priv->history);
 		break;
 	default:
 		return;
 	}
 	
-	if (url) {
-		yw_handle_url (window, url);
+	if (uri) {
+		window_handle_uri (window, uri);
 	}
 }
 
 static void
-yw_back_button_clicked (GtkWidget *button, YelpWindow *window)
+window_back_button_clicked (GtkWidget *button, YelpWindow *window)
 {
 	g_return_if_fail (GTK_IS_BUTTON (button));
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
-	yw_history_action (window, YELP_WINDOW_ACTION_BACK);
+	window_history_action (window, YELP_WINDOW_ACTION_BACK);
 }
 
 static void
-yw_forward_button_clicked (GtkWidget *button, YelpWindow *window)
+window_forward_button_clicked (GtkWidget *button, YelpWindow *window)
 {
 	g_return_if_fail (GTK_IS_BUTTON (button));
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
-	yw_history_action (window, YELP_WINDOW_ACTION_FORWARD);
+	window_history_action (window, YELP_WINDOW_ACTION_FORWARD);
 }
 
 static void
-yw_home_button_clicked (GtkWidget *button, YelpWindow *window)
+window_home_button_clicked (GtkWidget *button, YelpWindow *window)
 {
-	g_return_if_fail (YELP_IS_WINDOW (window));
-
-	yelp_history_goto (window->priv->history, "toc:");
+	YelpURI *uri;
 	
-	yelp_view_toc_open_url (YELP_VIEW_TOC (window->priv->toc_view),
-				"toc:");
+	g_return_if_fail (YELP_IS_WINDOW (window));
+	
+	uri = yelp_uri_new ("toc:");
+
+	yelp_history_goto (window->priv->history, uri);
+	yelp_view_toc_open_uri (window->priv->toc_view, uri);
+
+	yelp_uri_unref (uri);
 
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook),
 				       PAGE_TOC_VIEW);
 }
 
 static void
-yw_index_button_clicked (GtkWidget *button, YelpWindow *window)
+window_index_button_clicked (GtkWidget *button, YelpWindow *window)
 {
+	YelpURI *uri;
+	
 	g_return_if_fail (YELP_IS_WINDOW (window));
 
+	uri = yelp_uri_new ("index:");
+	yelp_history_goto (window->priv->history, uri);
+	yelp_uri_unref (uri);
+	
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (window->priv->notebook),
 				       PAGE_INDEX_VIEW);
 }
 
 static void
-yw_new_window_cb (gpointer data, guint section, GtkWidget *widget)
+window_new_window_cb (gpointer data, guint section, GtkWidget *widget)
 {
 	g_return_if_fail (YELP_IS_WINDOW (data));
 
@@ -528,33 +528,33 @@ yw_new_window_cb (gpointer data, guint section, GtkWidget *widget)
 }
 
 static void
-yw_close_window_cb (gpointer   data,
-		    guint      section,
-		    GtkWidget *widget)
+window_close_window_cb (gpointer   data,
+			guint      section,
+			GtkWidget *widget)
 {
 	gtk_widget_destroy (GTK_WIDGET (data));
 }
 
 static void
-yw_history_go_cb (gpointer data, guint section, GtkWidget *widget)
+window_history_go_cb (gpointer data, guint section, GtkWidget *widget)
 {
-	yw_history_action (data, section);
+	window_history_action (data, section);
 }
 
 static void
-yw_go_home_cb (gpointer data, guint section, GtkWidget *widget)
+window_go_home_cb (gpointer data, guint section, GtkWidget *widget)
 {
-	yw_home_button_clicked (NULL, YELP_WINDOW (data));
+	window_home_button_clicked (NULL, YELP_WINDOW (data));
 }
 
 static void
-yw_go_index_cb (gpointer data, guint section, GtkWidget *widget)
+window_go_index_cb (gpointer data, guint section, GtkWidget *widget)
 {
-	yw_index_button_clicked (NULL, YELP_WINDOW (data));
+	window_index_button_clicked (NULL, YELP_WINDOW (data));
 }
 
 static void
-yw_about_cb (gpointer data, guint section, GtkWidget *widget)
+window_about_cb (gpointer data, guint section, GtkWidget *widget)
 {
 	GtkWidget   *about;
 	const gchar *authors[] = { 
@@ -578,7 +578,7 @@ yw_about_cb (gpointer data, guint section, GtkWidget *widget)
 }
 
 static GtkWidget *
-yw_create_toolbar (YelpWindow *window)
+window_create_toolbar (YelpWindow *window)
 {
 	YelpWindowPriv  *priv;
 	GtkWidget       *toolbar;
@@ -609,11 +609,11 @@ yw_create_toolbar (YelpWindow *window)
 
 	gconf_client_notify_add (conf_client, 
 				 "/desktop/gnome/interface/toolbar_style",
-				 yw_toolbar_style_changed_cb,
+				 window_toolbar_style_changed_cb,
 				 toolbar, NULL, NULL);
 	
 /* 	g_signal_connect(toolbar, "destroy", */
-/* 			 G_CALLBACK(yw_remove_notify_cb), */
+/* 			 G_CALLBACK(window_remove_notify_cb), */
 /* 			 GINT_TO_POINTER(notify_id)); */
 
 	gtk_toolbar_set_style (GTK_TOOLBAR (toolbar), style);
@@ -625,7 +625,7 @@ yw_create_toolbar (YelpWindow *window)
 						     _("Back"),
 						     _("Show previous page in history"),
 						     NULL, icon, 
-						     G_CALLBACK (yw_back_button_clicked),
+						     G_CALLBACK (window_back_button_clicked),
 						     window);
 	
 	gtk_widget_set_sensitive (priv->back_button, FALSE);
@@ -637,7 +637,7 @@ yw_create_toolbar (YelpWindow *window)
 							_("Forward"),
 							_("Show next page in history"),
 							NULL, icon,
-							G_CALLBACK (yw_forward_button_clicked),
+							G_CALLBACK (window_forward_button_clicked),
 							window);
 
 	gtk_widget_set_sensitive (priv->forward_button, FALSE);
@@ -651,7 +651,7 @@ yw_create_toolbar (YelpWindow *window)
 					  _("Home"),
 					  _("Go to home view"), 
 					  NULL, icon,
-					  G_CALLBACK (yw_home_button_clicked),
+					  G_CALLBACK (window_home_button_clicked),
 					  window);
 
 	if (priv->index) {
@@ -662,7 +662,7 @@ yw_create_toolbar (YelpWindow *window)
 						  _("Index"),
 						  _("Search in the index"), 
 						  NULL, icon,
-						  G_CALLBACK (yw_index_button_clicked),
+						  G_CALLBACK (window_index_button_clicked),
 						  window);
 		
 	}
@@ -671,10 +671,10 @@ yw_create_toolbar (YelpWindow *window)
 }
 
 static void
-yw_toolbar_style_changed_cb (GConfClient *client,
-			     guint        cnxn_id,
-			     GConfEntry  *entry,
-			     gpointer     data)
+window_toolbar_style_changed_cb (GConfClient *client,
+				 guint        cnxn_id,
+				 GConfEntry  *entry,
+				 gpointer     data)
 {
         GtkToolbarStyle  style   = GTK_TOOLBAR_BOTH;
         GtkToolbar      *toolbar = GTK_TOOLBAR(data);
@@ -696,7 +696,7 @@ yw_toolbar_style_changed_cb (GConfClient *client,
 
 #if 0
 static void
-yw_remove_notify_cb (GtkObject *obj, gpointer data)
+window_remove_notify_cb (GtkObject *obj, gpointer data)
 {
         GConfClient *conf_client;
         guint        notify_id;
@@ -720,25 +720,28 @@ yelp_window_new (GNode *doc_tree, GList *index)
 	priv->doc_tree = doc_tree;
 	priv->index    = index;
 
- 	priv->toc_view    = yelp_view_toc_new (doc_tree);
+ 	priv->toc_view     = yelp_view_toc_new (doc_tree);
  	priv->content_view = yelp_view_content_new (doc_tree);
 
 	if (priv->index) {
 		priv->index_view = yelp_view_index_new (index);
 
-		g_signal_connect (priv->index_view, "url_selected",
-				  G_CALLBACK (yw_url_selected_cb),
+		g_signal_connect (priv->index_view, "uri_selected",
+				  G_CALLBACK (window_uri_selected_cb),
 				  window);
 	} 
 
-	g_signal_connect (priv->toc_view, "url_selected",
-			  G_CALLBACK (yw_url_selected_cb),
-			  window);
-	g_signal_connect (priv->content_view, "url_selected",
-			  G_CALLBACK (yw_url_selected_cb),
+	g_signal_connect (priv->toc_view, "uri_selected",
+			  G_CALLBACK (window_uri_selected_cb),
 			  window);
 
-	yw_populate (window);
+	g_signal_connect (priv->content_view, "uri_selected",
+			  G_CALLBACK (window_uri_selected_cb),
+			  window);
+
+	window_populate (window);
+
+	yelp_window_open_uri (window, "toc:");
 
         return GTK_WIDGET (window);
 }
@@ -748,15 +751,22 @@ yelp_window_open_uri (YelpWindow  *window,
 		      const gchar *str_uri)
 {
 	YelpWindowPriv *priv;
+	YelpURI        *uri;
 	
 	g_return_if_fail (YELP_IS_WINDOW (window));
 	
 	priv = window->priv;
 
-	yw_handle_url (window, str_uri);
+	uri = yelp_uri_new (str_uri);
+
+	if (yelp_uri_exists (uri)) {
+		window_handle_uri (window, uri);
+	}
+	
+	yelp_uri_unref (uri);
 }
 
-const gchar *
+YelpURI *
 yelp_window_get_current_uri (YelpWindow *window)
 {
 	g_return_val_if_fail (YELP_IS_WINDOW (window), NULL);
